@@ -100,6 +100,16 @@ static const char *get_filename(const bool recording)
     return devname;
 }
 
+static const char *AudioFormatString(SDL_AudioFormat fmt)
+{
+    const char *str = SDL_GetAudioFormatName(fmt);
+    SDL_assert(str);
+    if (SDL_strncmp(str, "SDL_AUDIO_", 10) == 0) {
+        str += 10;  // so we return "S8" instead of "SDL_AUDIO_S8", etc.
+    }
+    return str;
+}
+
 static bool DISKAUDIO_OpenDevice(SDL_AudioDevice *device)
 {
     bool recording = device->recording;
@@ -136,7 +146,9 @@ static bool DISKAUDIO_OpenDevice(SDL_AudioDevice *device)
     }
 
     SDL_LogCritical(SDL_LOG_CATEGORY_AUDIO, "You are using the SDL disk i/o audio driver!");
-    SDL_LogCritical(SDL_LOG_CATEGORY_AUDIO, " %s file [%s].", recording ? "Reading from" : "Writing to", fname);
+    SDL_LogCritical(SDL_LOG_CATEGORY_AUDIO, " %s file [%s], format=%s channels=%d freq=%d.",
+                    recording ? "Reading from" : "Writing to", fname,
+                    AudioFormatString(device->spec.format), device->spec.channels, device->spec.freq);
 
     return true;  // We're ready to rock and roll. :-)
 }
