@@ -24,12 +24,3 @@
 /* Return true if the HIDAPI should ignore a device during enumeration */
 extern bool SDL_HIDAPI_ShouldIgnoreDevice(int bus_type, Uint16 vendor_id, Uint16 product_id, Uint16 usage_page, Uint16 usage);
 
-#ifdef SDL_JOYSTICK_HIDAPI
-#ifdef HAVE_LIBUSB
-#define HAVE_ENABLE_GAMECUBE_ADAPTORS
-#endif
-
-#ifdef HAVE_ENABLE_GAMECUBE_ADAPTORS
-extern void SDL_EnableGameCubeAdaptors(void);
-#endif
-#endif /* SDL_JOYSTICK_HIDAPI */
