@@ -84,7 +84,7 @@ static SDL_GetPowerInfo_Impl implementations[] = {
 SDL_PowerState SDL_GetPowerInfo(int *seconds, int *percent)
 {
 #ifndef SDL_POWER_DISABLED
-    const int total = sizeof(implementations) / sizeof(implementations[0]);
+    const int total = SDL_arraysize(implementations);
     SDL_PowerState result = SDL_POWERSTATE_UNKNOWN;
     int i;
 #endif
