@@ -23,13 +23,10 @@
 
 #include "../SDL_tray_utils.h"
 #include "../../core/windows/SDL_windows.h"
-#include "../../video/windows/SDL_windowswindow.h"
 
 #include <windowsx.h>
 #include <shellapi.h>
 
-#include "../../video/windows/SDL_surface_utils.h"
-
 #ifndef NOTIFYICON_VERSION_4
 #define NOTIFYICON_VERSION_4 4
 #endif
@@ -111,11 +108,20 @@ LRESULT CALLBACK TrayWindowProc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lPar
         return DefWindowProc(hwnd, uMsg, wParam, lParam);
     }
 
+    static UINT s_taskbarRestart = 0;
+    if (s_taskbarRestart == 0) {
+        s_taskbarRestart = RegisterWindowMessage(TEXT("TaskbarCreated"));
+    }
+    if (uMsg == s_taskbarRestart) {
+        Shell_NotifyIconW(NIM_ADD, &tray->nid);
+        Shell_NotifyIconW(NIM_SETVERSION, &tray->nid);
+    }
+
     switch (uMsg) {
         case WM_TRAYICON:
             if (LOWORD(lParam) == WM_CONTEXTMENU || LOWORD(lParam) == WM_LBUTTONUP) {
                 SetForegroundWindow(hwnd);
-                
+
                 if (tray->menu) {
                     TrackPopupMenu(tray->menu->hMenu, TPM_BOTTOMALIGN | TPM_RIGHTALIGN, GET_X_LPARAM(wParam), GET_Y_LPARAM(wParam), 0, hwnd, NULL);
                 }
@@ -216,6 +222,51 @@ void SDL_UpdateTrays(void)
 {
 }
 
+static void SDL_PropTrayCleanupCb(void *userdata, void *cls)
+{
+    WNDCLASSEX wcex;
+
+    wcex.hIcon = NULL;
+    wcex.hIconSm = NULL;
+    HINSTANCE h = GetModuleHandle(NULL);
+    if (GetClassInfoEx(h, cls, &wcex)) {
+        UnregisterClass(cls, h);
+    }
+}
+
+static bool SDL_RegisterTrayClass(LPCWSTR className)
+{
+    SDL_PropertiesID props = SDL_GetGlobalProperties();
+    if (!props) {
+        return false;
+    }
+    if (SDL_GetPointerProperty(props, SDL_PROP_TRAY_CLEANUP, NULL) != NULL) {
+        return true;
+    }
+
+    WNDCLASSEX wcex;
+
+    wcex.cbSize = sizeof(WNDCLASSEX);
+    wcex.hCursor = NULL;
+    wcex.hIcon = NULL;
+    wcex.hIconSm = NULL;
+    wcex.lpszMenuName = NULL;
+    wcex.lpszClassName = className;
+    wcex.style = 0;
+    wcex.hbrBackground = NULL;
+    wcex.lpfnWndProc = TrayWindowProc;
+    wcex.hInstance = NULL;
+    wcex.cbClsExtra = 0;
+    wcex.cbWndExtra = 0;
+
+    if (!RegisterClassEx(&wcex)) {
+        return SDL_SetError("Couldn't register tray class");
+    }
+
+    SDL_SetPointerPropertyWithCleanup(props, SDL_PROP_TRAY_CLEANUP, (void *)wcex.lpszClassName, SDL_PropTrayCleanupCb, NULL);
+    return true;
+}
+
 SDL_Tray *SDL_CreateTray(SDL_Surface *icon, const char *tooltip)
 {
     if (!SDL_IsMainThread()) {
@@ -230,8 +281,12 @@ SDL_Tray *SDL_CreateTray(SDL_Surface *icon, const char *tooltip)
     }
 
     tray->menu = NULL;
-    tray->hwnd = CreateWindowEx(0, TEXT("Message"), NULL, 0, 0, 0, 0, 0, HWND_MESSAGE, NULL, NULL, NULL);
-    SetWindowLongPtr(tray->hwnd, GWLP_WNDPROC, (LONG_PTR) TrayWindowProc);
+    if (!SDL_RegisterTrayClass(TEXT("SDL_TRAY"))) {
+        SDL_SetError("Failed to register SDL_TRAY window class");
+        return NULL;
+    }
+    tray->hwnd = CreateWindowEx(0, TEXT("SDL_TRAY"), NULL, WS_OVERLAPPEDWINDOW,
+                                CW_USEDEFAULT, 0, CW_USEDEFAULT, 0, NULL, NULL, NULL, NULL);
 
     WIN_UpdateDarkModeForHWND(tray->hwnd);
 
@@ -247,7 +302,7 @@ SDL_Tray *SDL_CreateTray(SDL_Surface *icon, const char *tooltip)
     SDL_free(tooltipw);
 
     if (icon) {
-        tray->nid.hIcon = CreateIconFromSurface(icon);
+        tray->nid.hIcon = WIN_CreateIconFromSurface(icon);
 
         if (!tray->nid.hIcon) {
             tray->nid.hIcon = load_default_icon();
@@ -280,7 +335,7 @@ void SDL_SetTrayIcon(SDL_Tray *tray, SDL_Surface *icon)
     }
 
     if (icon) {
-        tray->nid.hIcon = CreateIconFromSurface(icon);
+        tray->nid.hIcon = WIN_CreateIconFromSurface(icon);
 
         if (!tray->nid.hIcon) {
             tray->nid.hIcon = load_default_icon();
@@ -314,7 +369,7 @@ void SDL_SetTrayTooltip(SDL_Tray *tray, const char *tooltip)
 
 SDL_TrayMenu *SDL_CreateTrayMenu(SDL_Tray *tray)
 {
-    if (!SDL_ObjectValid(tray, SDL_OBJECT_TYPE_TRAY)) {
+    CHECK_PARAM(!SDL_ObjectValid(tray, SDL_OBJECT_TYPE_TRAY)) {
         SDL_InvalidParamError("tray");
         return NULL;
     }
@@ -334,7 +389,7 @@ SDL_TrayMenu *SDL_CreateTrayMenu(SDL_Tray *tray)
 
 SDL_TrayMenu *SDL_GetTrayMenu(SDL_Tray *tray)
 {
-    if (!SDL_ObjectValid(tray, SDL_OBJECT_TYPE_TRAY)) {
+    CHECK_PARAM(!SDL_ObjectValid(tray, SDL_OBJECT_TYPE_TRAY)) {
         SDL_InvalidParamError("tray");
         return NULL;
     }
@@ -344,7 +399,7 @@ SDL_TrayMenu *SDL_GetTrayMenu(SDL_Tray *tray)
 
 SDL_TrayMenu *SDL_CreateTraySubmenu(SDL_TrayEntry *entry)
 {
-    if (!entry) {
+    CHECK_PARAM(!entry) {
         SDL_InvalidParamError("entry");
         return NULL;
     }
@@ -359,7 +414,7 @@ SDL_TrayMenu *SDL_CreateTraySubmenu(SDL_TrayEntry *entry)
 
 SDL_TrayMenu *SDL_GetTraySubmenu(SDL_TrayEntry *entry)
 {
-    if (!entry) {
+    CHECK_PARAM(!entry) {
         SDL_InvalidParamError("entry");
         return NULL;
     }
@@ -369,7 +424,7 @@ SDL_TrayMenu *SDL_GetTraySubmenu(SDL_TrayEntry *entry)
 
 const SDL_TrayEntry **SDL_GetTrayEntries(SDL_TrayMenu *menu, int *count)
 {
-    if (!menu) {
+    CHECK_PARAM(!menu) {
         SDL_InvalidParamError("menu");
         return NULL;
     }
@@ -422,12 +477,12 @@ void SDL_RemoveTrayEntry(SDL_TrayEntry *entry)
 
 SDL_TrayEntry *SDL_InsertTrayEntryAt(SDL_TrayMenu *menu, int pos, const char *label, SDL_TrayEntryFlags flags)
 {
-    if (!menu) {
+    CHECK_PARAM(!menu) {
         SDL_InvalidParamError("menu");
         return NULL;
     }
 
-    if (pos < -1 || pos > menu->nEntries) {
+    CHECK_PARAM(pos < -1 || pos > menu->nEntries) {
         SDL_InvalidParamError("pos");
         return NULL;
     }
@@ -553,7 +608,7 @@ void SDL_SetTrayEntryLabel(SDL_TrayEntry *entry, const char *label)
 
 const char *SDL_GetTrayEntryLabel(SDL_TrayEntry *entry)
 {
-    if (!entry) {
+    CHECK_PARAM(!entry) {
         SDL_InvalidParamError("entry");
         return NULL;
     }
@@ -636,7 +691,7 @@ void SDL_ClickTrayEntry(SDL_TrayEntry *entry)
 
 SDL_TrayMenu *SDL_GetTrayEntryParent(SDL_TrayEntry *entry)
 {
-    if (!entry) {
+    CHECK_PARAM(!entry) {
         SDL_InvalidParamError("entry");
         return NULL;
     }
@@ -646,7 +701,7 @@ SDL_TrayMenu *SDL_GetTrayEntryParent(SDL_TrayEntry *entry)
 
 SDL_TrayEntry *SDL_GetTrayMenuParentEntry(SDL_TrayMenu *menu)
 {
-    if (!menu) {
+    CHECK_PARAM(!menu) {
         SDL_InvalidParamError("menu");
         return NULL;
     }
@@ -656,7 +711,7 @@ SDL_TrayEntry *SDL_GetTrayMenuParentEntry(SDL_TrayMenu *menu)
 
 SDL_Tray *SDL_GetTrayMenuParentTray(SDL_TrayMenu *menu)
 {
-    if (!menu) {
+    CHECK_PARAM(!menu) {
         SDL_InvalidParamError("menu");
         return NULL;
     }
