@@ -68,11 +68,13 @@ static void DUMMY_SetWindowSize(SDL_VideoDevice *_this, SDL_Window *window)
 
 // DUMMY driver bootstrap functions
 
-static bool DUMMY_Available(const char *drivername)
+static bool DUMMY_Available(const char *enable_hint)
 {
     const char *hint = SDL_GetHint(SDL_HINT_VIDEO_DRIVER);
-    if (hint && SDL_strstr(hint, drivername) != NULL) {
-        return true;
+    if (hint) {
+        if (SDL_strcmp(hint, enable_hint) == 0) {
+            return true;
+        }
     }
     return false;
 }
@@ -147,6 +149,13 @@ VideoBootStrap DUMMY_evdev_bootstrap = {
     false
 };
 
+#else
+
+static bool DUMMY_SetRelativeMouseMode(bool enabled)
+{
+    return true;
+}
+
 #endif // SDL_INPUT_LINUXEV
 
 bool DUMMY_VideoInit(SDL_VideoDevice *_this)
@@ -164,6 +173,8 @@ bool DUMMY_VideoInit(SDL_VideoDevice *_this)
 
 #ifdef SDL_INPUT_LINUXEV
     SDL_EVDEV_Init();
+#else
+    SDL_GetMouse()->SetRelativeMouseMode = DUMMY_SetRelativeMouseMode;
 #endif
 
     // We're done!
