@@ -114,10 +114,11 @@ static SDL_VideoDevice *PSP_Create(void)
     device->HasScreenKeyboardSupport = PSP_HasScreenKeyboardSupport;
     device->ShowScreenKeyboard = PSP_ShowScreenKeyboard;
     device->HideScreenKeyboard = PSP_HideScreenKeyboard;
-    device->IsScreenKeyboardShown = PSP_IsScreenKeyboardShown;
 
     device->PumpEvents = PSP_PumpEvents;
 
+    device->device_caps = VIDEO_DEVICE_CAPS_FULLSCREEN_ONLY;
+
     return device;
 }
 
@@ -456,6 +457,8 @@ void PSP_ShowScreenKeyboard(SDL_VideoDevice *_this, SDL_Window *window, SDL_Prop
 
     sceUtilityOskInitStart(&params);
 
+    SDL_SendScreenKeyboardShown();
+
     while(!done) {
         sceGuStart(GU_DIRECT, list);
         sceGuClearColor(0);
@@ -487,13 +490,12 @@ void PSP_ShowScreenKeyboard(SDL_VideoDevice *_this, SDL_Window *window, SDL_Prop
         text_string[i] = outtext[i];
     }
     SDL_SendKeyboardText((const char *) text_string);
+
+    SDL_SendScreenKeyboardHidden();
 }
+
 void PSP_HideScreenKeyboard(SDL_VideoDevice *_this, SDL_Window *window)
 {
 }
-bool PSP_IsScreenKeyboardShown(SDL_VideoDevice *_this, SDL_Window *window)
-{
-    return false;
-}
 
 #endif // SDL_VIDEO_DRIVER_PSP
