@@ -27,13 +27,13 @@
 #ifdef SDL_VIDEO_DRIVER_X11_XSYNC
 
 extern void X11_InitXsync(SDL_VideoDevice *_this);
-extern int X11_XsyncIsInitialized(void);
-int X11_InitResizeSync(SDL_Window *window);
-void X11_TermResizeSync(SDL_Window *window);
-void X11_HandleSyncRequest(SDL_Window *window, XClientMessageEvent *event);
-void X11_HandleConfigure(SDL_Window *window, XConfigureEvent *event);
-void X11_HandlePresent(SDL_Window *window);
+extern bool X11_XsyncIsInitialized(void);
+extern bool X11_InitResizeSync(SDL_Window *window);
+extern void X11_TermResizeSync(SDL_Window *window);
+extern void X11_HandleSyncRequest(SDL_Window *window, XClientMessageEvent *event);
+extern void X11_HandleConfigure(SDL_Window *window, XConfigureEvent *event);
+extern void X11_HandlePresent(SDL_Window *window);
 
-#endif /* SDL_VIDEO_DRIVER_X11_XSYNC */
+#endif // SDL_VIDEO_DRIVER_X11_XSYNC
 
-#endif /* SDL_x11xsync_h_ */
+#endif // SDL_x11xsync_h_
